@@ -1,5 +1,3 @@
-context("assertions about assertions in assertion.R")
-
 # just some set up
 a <- 1
 alist <- list(a=c(1,2,3), b=c(4,5,6))
@@ -197,8 +195,11 @@ test_that("verify breaks appropriately", {
   expect_error(verify(mtcars), "argument \"expr\" is missing, with no default")
   expect_error(verify(MTCARS, 2 > 1),
                "object 'MTCARS' not found")
-  expect_warning(verify(mtcars, 1),
-                 "coercing argument of type 'double' to logical")
+  expect_warning(expect_warning(
+    verify(mtcars, 1),
+    "coercing argument of type 'double' to logical"),
+    "The result of evaluating '1' is not a logical vector"
+  )
   expect_error(suppressWarnings(verify(mtcars, "1")),
                "missing value where TRUE/FALSE needed")
   expect_error(verify(mtcars, 1 > 0, "tree"), "could not find function \"success_fun\"")
@@ -262,8 +263,8 @@ test_that("assert returns TRUE if verification passes (w/ `success_logical`)", {
   expect_true(assert(mtcars, not_na, vs, success_fun=success_logical))
   expect_true(assert(mtcars, not_na, mpg:carb, success_fun=success_logical))
   # lambdas
-  expect_true(assert(mtcars, function(x) x%%1==0, cyl, vs, am, gear, carb,
-                     success_fun=success_logical))
+  # expect_true(assert(mtcars, function(x) x%%1==0, cyl, vs, am, gear, carb,
+  #                    success_fun=success_logical))
   expect_true(assert(mtcars, function(x) if(x%%1!=0) return(FALSE), gear,
                      success_fun=success_logical))
   expect_true(assert(iris, function(x) nchar(as.character(x)) > 5, Species,
@@ -350,6 +351,17 @@ test_that("skip_chain_opts doesn't affect functionality outside chain for assert
                 "Column 'gear' violates assertion 'within_bounds\\(3.5, 4.5\\)' 20 times.*")
 })
 
+test_that("assert works with single row data.frames", {
+  single_row_data <- head(mtcars, 1)
+
+  expect_equal(assert(single_row_data, within_bounds(10,30), disp, error_fun = error_logical), FALSE)
+  expect_output(assert(single_row_data, within_bounds(10,30), disp, error_fun = just.show.error),
+               "Column 'disp' violates assertion 'within_bounds\\(10, 30\\)' 1 time.*")
+  expect_equal(assert(single_row_data, within_bounds(10,30), disp, mpg, error_fun = error_logical), FALSE)
+  expect_output(assert(single_row_data, within_bounds(10,30), disp, mpg, error_fun = just.show.error),
+               "Column 'disp' violates assertion 'within_bounds\\(10, 30\\)' 1 time.*")
+})
+
 ######################################
 
 
@@ -1324,23 +1336,18 @@ test_that("all assertions work with .data pronoun without chains", {
   expect_equal(verify(test.df, y <= 2), test.df)
 
   ## assert() ##
-  expect_equal(assert(test.df, within_bounds(-Inf, 2), .data$x), test.df)
-  expect_error(assert(test.df, within_bounds(2, Inf), .data$x))
-  # Cases where the name doesn't exist:
-  expect_error(assert(test.df, within_bounds(-Inf, 2), .data$y))
-  # Note that assert(test.df, within_bounds(-Inf, 2), y) would not work because
-  # assert relies on dplyr::select. Use !! varname
+  expect_equal(assert(test.df, within_bounds(-Inf, 2), x), test.df)
+  expect_output(
+    expect_error(assert(test.df, within_bounds(2, Inf), x)),
+    regexp="Column 'x' violates assertion 'within_bounds(2, Inf)' 2 times",
+    fixed=TRUE
+  )
 
   ## insist() ##
-  expect_equal(insist(test.df, within_n_sds(1), .data$x), test.df)
-  expect_output(insist(test.df, within_n_sds(0.1), .data$x,
+  expect_equal(insist(test.df, within_n_sds(1), x), test.df)
+  expect_output(insist(test.df, within_n_sds(0.1), x,
     error_fun = just.show.error),
     "Column 'x' violates assertion 'within_n_sds(0.1)' 2 times", fixed = TRUE)
-
-  # Cases where the name doesn't exist:
-  expect_error(insist(test.df, within_n_sds(1), .data$y))
-  # Note that insist(test.df, within_n_sds(1), y) would not work because
-  # insist relies on dplyr::select. Use !! y instead.
 })
 
 test_that("all assertions work with .data pronoun in chains", {
@@ -1350,36 +1357,28 @@ test_that("all assertions work with .data pronoun in chains", {
   ## verify() ##
   # Cases where the name exists:
   # Also test the logical versions here to make sure nothing too weird is happening.
-  expect_equal(test.df %>% verify(.data$x <= 2), test.df)
-  expect_true(test.df %>% verify(.data$x <= 2, success_fun = success_logical))
-  expect_output(test.df %>% verify(.data$x > 2, error_fun = just.show.error),
-                "verification [.data$x > 2] failed! (3 failures)", fixed = TRUE)
-  expect_false(test.df %>% verify(.data$x > 2, error_fun = error_logical))
+  expect_equal(test.df %>% verify(x <= 2), test.df)
+  expect_true(test.df %>% verify(x <= 2, success_fun = success_logical))
+  expect_output(test.df %>% verify(x > 2, error_fun = just.show.error),
+                "verification [x > 2] failed! (3 failures)", fixed = TRUE)
+  expect_false(test.df %>% verify(x > 2, error_fun = error_logical))
 
   # Cases where the name doesn't exist:
+  #expect_error(test.df %>% verify(y <= 2, error_fun = just.show.error))
   expect_error(test.df %>% verify(.data$y <= 2, error_fun = just.show.error))
   expect_equal(test.df %>% verify(y <= 2), test.df)
 
   ## assert() ##
-  expect_equal(test.df %>% assert(within_bounds(-Inf, 2), .data$x), test.df)
-  expect_output(test.df %>% assert(within_bounds(2, Inf), .data$x,
+  expect_equal(test.df %>% assert(within_bounds(-Inf, 2), x), test.df)
+  expect_output(test.df %>% assert(within_bounds(2, Inf), x,
     error_fun = just.show.error),
     "Column 'x' violates assertion 'within_bounds(2, Inf)' 2 times", fixed = TRUE)
-  # Cases where the name doesn't exist:
-  expect_error(test.df %>% assert(within_bounds(-Inf, 2), .data$y))
-  # Note that test.df %>% assert(within_bounds(-Inf, 2), y) would not work because
-  # assert relies on dplyr::select.
 
   ## insist() ##
-  expect_equal(test.df %>% insist(within_n_sds(1), .data$x), test.df)
-  expect_output(test.df %>% insist(within_n_sds(0.1), .data$x,
+  expect_equal(test.df %>% insist(within_n_sds(1), x), test.df)
+  expect_output(test.df %>% insist(within_n_sds(0.1), x,
     error_fun = just.show.error),
     "Column 'x' violates assertion 'within_n_sds(0.1)' 2 times", fixed = TRUE)
-
-  # Cases where the name doesn't exist:
-  expect_error(test.df %>% insist(within_n_sds(1), .data$y))
-  # Note that test.df %>% insist(within_n_sds(1), y) would not work because
-  # insist relies on dplyr::select.
 })
 
 
@@ -1602,3 +1601,15 @@ test_that("description is correctly stored and displayed in results", {
   )
 
 })
+
+test_that("handle predicates applied to the whole data, and not to subframe", {
+  a_tibble <- tibble::tibble(
+    a = c(0, 0),
+    b = c(0,0)
+  ) %>%
+    head( n = 0)
+
+  expect_silent({
+    assert(data = a_tibble, predicate = function(x) nrow(x) == 0L, dplyr::everything())
+  })
+})
